/**
 * M1_5BinaryStudy.java
 *
 * @author Antonio J. Nebro
 * @version 1.0
 */
package jmetal.experiments;

import java.util.logging.Level;
import java.util.logging.Logger;
import java.io.IOException;
import java.util.Properties;
import jmetal.base.Algorithm;
import jmetal.base.Problem;
import jmetal.experiments.settings.*;
import jmetal.experiments.util.RBoxplot;
import jmetal.experiments.util.RWilcoxon;
import jmetal.util.JMException;

/**
 * @author Antonio J. Nebro
 */
public class M1_5BinaryStudy extends Experiment {

  /**
   * Configures the algorithms in each independent run
   * @param problem The problem to solve
   * @param problemIndex
   * @throws ClassNotFoundException 
   */
  public void algorithmSettings(Problem problem, 
  		                          int problemIndex, 
  		                          Algorithm[] algorithm) throws ClassNotFoundException {
    try {
      int numberOfAlgorithms = algorithmNameList_.length;

      Properties[] parameters = new Properties[numberOfAlgorithms];

      for (int i = 0; i < numberOfAlgorithms; i++) {
        parameters[i] = new Properties();
      } // for

      if (!paretoFrontFile_[problemIndex].equals("")) {
        for (int i = 0; i < numberOfAlgorithms; i++)
          parameters[i].setProperty("paretoFrontFile_", paretoFrontFile_[problemIndex]);
        } // if

        algorithm[0] = new M1_5_NSGAII_BinarySettings(problem).configure(parameters[0]);
        algorithm[1] = new M1_5_ssNSGAII_BinarySettings(problem).configure(parameters[2]);  
        algorithm[2] = new M1_5_SPEA2_BinarySettings(problem).configure(parameters[1]);
        algorithm[3] = new M1_5_IBEA_BinarySettings(problem).configure(parameters[3]);
      } catch (IllegalArgumentException ex) {
      Logger.getLogger(M1_5BinaryStudy.class.getName()).log(Level.SEVERE, null, ex);
    } catch (IllegalAccessException ex) {
      Logger.getLogger(M1_5BinaryStudy.class.getName()).log(Level.SEVERE, null, ex);
    } catch  (JMException ex) {
      Logger.getLogger(M1_5BinaryStudy.class.getName()).log(Level.SEVERE, null, ex);
    }
  } // algorithmSettings

  /**
   * Main method
   * @param args
   * @throws JMException
   * @throws IOException
   */
  public static void main(String[] args) throws JMException, IOException {
    M1_5BinaryStudy exp = new M1_5BinaryStudy();

    exp.experimentName_ = "M1.5-BinaryStudy";
    exp.algorithmNameList_ = new String[]{"NSGAII", "ssNSGAII", "SPEA2", "IBEA"};
    exp.problemList_ = new String[]{"ZDT5"};
    exp.paretoFrontFile_ = new String[]{"M1.5.ZDT5.pf"};

    exp.indicatorList_   = new String[] {"HV", "SPREAD","GD" ,"IGD", "EPSILON"} ;

    int numberOfAlgorithms = exp.algorithmNameList_.length;

    exp.experimentBaseDirectory_ = "C:/Users/prad/Desktop/trunk/data/results/" + exp.experimentName_;
    exp.paretoFrontDirectory_ = "C:/Users/prad/Desktop/trunk/data/M1.5fronts";

    exp.algorithmSettings_ = new Settings[numberOfAlgorithms];

    exp.independentRuns_ = 51;

    // Run the experiments
    int numberOfThreads ;
    exp.runExperiment(numberOfThreads = 1) ;

    // Generate latex tables
    exp.generateLatexTables() ;

    // Configure the R scripts to be generated
    int rows  ;
    int columns  ;
    String prefix ;
    String [] problems ;
    boolean notch ;

    // Configuring scripts for CTP
    rows = 1 ;
    columns = 1 ;
    prefix = new String("ZDT");
    problems = new String[]{"ZDT5"} ;
    
    exp.generateRBoxplotScripts(rows, columns, problems, prefix, notch = false, exp) ;
    exp.generateRWilcoxonScripts(problems, prefix, exp) ;
    
  } // main
} // M1_5BinaryStudy


