package jmetal.util.io;


import java.io.*;
import java.util.*;
import jmetal.base.Solution;
import jmetal.base.SolutionSet;
import jmetal.util.MProperParetoOptimality;


public class MPPOConverter {

	/**
	 * @param args
	 */
	public static void main(String[] args) throws FileNotFoundException{
		// TODO Auto-generated method stub
	
		//Pathname of source and destination, value of M used for MPPO
		
		File folder = new File("data/paretoFronts");
		String[] files = folder.list();
		
		for(int k = 0 ; k < files.length ; k++){
		
		String filename = files[k];
		System.out.println("Converting: " + files[k]);
		
		String sourcepath = "data/paretoFronts/" + filename;
		String destinationpath = "data/M5fronts/" + "M5_" + filename; 
		
		
		File source = new File(sourcepath);
		File destination = new File(destinationpath);
		double M = 5.0;
		
		try {
			// Initializes the input reader
			FileInputStream fis = new FileInputStream(source);
			InputStreamReader isr = new InputStreamReader(fis);
			BufferedReader br = new BufferedReader(isr);
			
			// Read first line, initialize list of solutions and store number of objectives
			String line = br.readLine();
			//String[] current = line.split( "\t" );
			char[] c = {'\u0020'};
			String s = new String(c);
			String[] current = line.split( s );
			
			double currentValue;
			int nObj = current.length;
			
			LinkedList<Solution> list = new LinkedList<Solution>();
			Solution sol = new Solution(nObj);
			
			for(int i = 0 ; i < nObj ; i++){
				currentValue = Double.valueOf(current[i]).doubleValue();
				sol.setObjective(i, currentValue);
			}
			
			list.add(sol);
			
			// Read solutions line by line
			
			while(br.ready()){
				line = br.readLine();
				//current = line.split( "\t" );
				current = line.split( s );
				sol = new Solution(nObj);
				
				for(int i = 0 ; i < nObj ; i++){
					currentValue = Double.valueOf(current[i]).doubleValue();
					sol.setObjective(i, currentValue);
				}
				
				list.add(sol);
			}
			
			SolutionSet solutionset = new SolutionSet( list.size() );
			
			// Write solutions in list to SolutionSet
			Iterator it = list.iterator();
			while(it.hasNext()){
				sol = (Solution) it.next();
				solutionset.add(sol);
			}
			
			// Do MPPO Sorting
			MProperParetoOptimality mppo = new MProperParetoOptimality(M);
			SolutionSet results[] = mppo.create(solutionset);
			
			br.close();
			
			try{
				// Initialize output writer
				FileOutputStream fos   = new FileOutputStream(destination);
			    OutputStreamWriter osw = new OutputStreamWriter(fos);
			    BufferedWriter bw      = new BufferedWriter(osw);
			    for (int i = 0; i < results[0].size(); i++) {
			        bw.write(results[0].get(i).toString());
			        bw.newLine();
			    }
			    
			    bw.close();
			} catch(IOException e){
				System.out.println("Writing MPPO Set failed.");
			}
			
		} catch (IOException e){
			System.out.println("Source file could not be found.");
		}
		}
	}

}
